" Filename:    spec.vim
" Purpose:     Vim syntax file
" Language:    SPEC: Build/install scripts for Linux RPM packages
" Maintainer:  Donovan Rebbechi elflord@pegasus.rutgers.edu
" URL:         http://pegasus.rutgers.edu/~elflord/vim/syntax/spec.vim
" Last Change: Sun Sep 28 22:10:20 MSD 2003 <raorn@altlinux.ru>

" For version 5.x: Clear all syntax items
" For version 6.x: Quit when a syntax file was already loaded
if version < 600
  syntax clear
elseif exists("b:current_syntax")
  finish
endif

syn sync minlines=1000

syn match specSpecialChar contained '[][!$()\\|>^;:{}]'
syn match specColon       contained ':'
syn match specPercent     contained '%'

syn match specVariables   contained '\$\h\w*' contains=specSpecialVariablesNames,specSpecialChar
syn match specVariables   contained '\${\w*}' contains=specSpecialVariablesNames,specSpecialChar

syn match specMacroIdentifier contained '%\h\w*' contains=specMacroNameLocal,specMacroNameOther,specPercent
syn match specMacroIdentifier contained '%{\w*}' contains=specMacroNameLocal,specMacroNameOther,specPercent,specSpecialChar

syn match specSpecialVariables contained '\$[0-9]\|\${[0-9]}'
syn match specCommandOpts      contained '\s\(-\w\+\|--\w[a-zA-Z_-]\+\)'ms=s+1
syn match specComment '^\s*#.*$'


syn case match


"matches with no highlight
syn match specNoNumberHilite 'X11\|X11R6\|[a-zA-Z]*\.\d\|[a-zA-Z][-/]\d'
syn match specManpageFile '[a-zA-Z]\.[1-8]'

"Day, Month and most used license acronyms
syn keyword specLicense contained GPL LGPL BSD MIT GNU
syn keyword specWeekday contained Mon Tue Wed Thu Fri Sat Sun
syn keyword specMonth   contained Jan Feb Mar Apr Jun Jul Aug Sep Oct Nov Dec
syn keyword specMonth   contained January February March April May June July August September October November December

"#, @, www
syn match specNumber '\(^-\=\|[ \t]-\=\|-\)[0-9.-]*[0-9]'
syn match specAltNumber '\(^-\=\|[ \t]-\=\|-\)[0-9.-]*alt[0-9.]*\S*'
syn match specEmail contained "<\=\<[A-Za-z0-9_.-]\+@\([A-Za-z0-9_-]\+\.\)\+[A-Za-z]\+\>>\="
syn match specURL      contained '\<\(\(https\{0,1}\|ftp\)://\|\(www[23]\{0,1}\.\|ftp\.\)\)[A-Za-z0-9._/~:,#-]\+\>'
syn match specURLMacro contained '\<\(\(https\{0,1}\|ftp\)://\|\(www[23]\{0,1}\.\|ftp\.\)\)[A-Za-z0-9._/~:,#%{}-]\+\>' contains=specMacroIdentifier

"TODO take specSpecialVariables out of the cluster for the sh* contains (ALLBUT)
"Special system directories
syn match specListedFilesPrefix contained '/\(usr\|local\|opt\|X11R6\|X11\)/'me=e-1
syn match specListedFilesBin    contained '/s\=bin/'me=e-1
syn match specListedFilesLib    contained '/\(lib\|include\)/'me=e-1
syn match specListedFilesDoc    contained '/\(man\d*\|doc\|info\)\>'
syn match specListedFilesEtc    contained '/etc/'me=e-1
syn match specListedFilesShare  contained '/share/'me=e-1
syn cluster specListedFiles contains=specListedFilesBin,specListedFilesLib,specListedFilesDoc,specListedFilesEtc,specListedFilesShare,specListedFilesPrefix,specVariables,specSpecialChar

"specComands
syn match   specConfigure  contained '\(\(\.\./\)*\.\)\{0,1}\./configure'
syn match   specTarCommand contained '\<tar\s\+[cxvpzIf]\{,5\}\s*'
syn keyword specCommandSpecial contained root
syn keyword specCommand        contained make xmkmf mkdir chmod ln find xargs subst sed rm strip moc echo grep ls rm mv mkdir install cp pwd cat tail then else elif cd gzip rmdir ln eval export touch
syn cluster specCommands contains=specCommand,specTarCommand,specConfigure,specCommandSpecial

"frequently used rpm env vars
syn keyword specSpecialVariablesNames contained RPM_ARCH RPM_BUILD_DIR RPM_BUILD_NCPUS RPM_BUILD_ROOT RPM_CLEANUP_METHOD RPM_CLEANUP_SKIPLIST RPM_CLEANUP_TOPDIR RPM_COMPRESS_METHOD RPM_COMPRESS_SKIPLIST RPM_COMPRESS_TOPDIR RPM_DOC_DIR RPM_FINDPROV_DEFAULT_METHOD RPM_FINDPROV_METHOD RPM_FINDPROV_SKIPLIST RPM_FINDPROV_TOPDIR RPM_FINDREQ_DEFAULT_METHOD RPM_FINDREQ_METHOD RPM_FINDREQ_SKIPLIST RPM_FINDREQ_TOPDIR RPM_FIXUP_METHOD RPM_FIXUP_SKIPLIST RPM_FIXUP_TOPDIR RPM_OPT_FLAGS RPM_OS RPM_PACKAGE_NAME RPM_PACKAGE_RELEASE RPM_PACKAGE_VERSION RPM_PERL_LIB_PATH RPM_PYTHON RPM_SOURCE_DIR RPM_STRIP_METHOD RPM_STRIP_SKIPLIST RPM_STRIP_TOPDIR RPM_VERIFY_ELF_METHOD RPM_VERIFY_ELF_SKIPLIST RPM_VERIFY_ELF_TOPDIR LDFLAGS CC CC_FLAGS CPPNAME CFLAGS CXX CXXFLAGS CPPFLAGS

"valid macro names from /usr/lib/rpm/macros
syn keyword specMacroNameOther contained amd buildroot buildsubdir distribution disturl intel ix86 nil optflags optflags_core optflags_debug optflags_default optflags_fastmath optflags_kernel optflags_nocpp optflags_notraceback optflags_optimization optflags_shared optflags_strict optflags_warnings prefix vendor serial name version release epoch
syn match   specMacroNameOther contained '\<\(PATCH\|SOURCE\)\d*\>'

"valid _macro names from /usr/lib/rpm/macros
syn keyword specMacroNameLocal contained _arch _bindir _build _build_alias _build_arch _build_cpu _builddir _build_name_fmt _build_os _buildshell _build_vendor _bzip2bin _cachedir _cleanup_method _cleanup_skiplist _cleanup_topdir _compress_method _compress_skiplist _compress_topdir _configure_gettext _configure_target _datadir _dbapi _dbapi_rebuild _dbi_btconfig _dbi_config _dbi_config_Depends _dbi_config_Dirnames _dbi_config_Installtid _dbi_config_Packages _dbi_config_Provideversion _dbi_config_Removetid _dbi_config_Requireversion _dbi_htconfig _dbi_tags _dbpath _dbpath_rebuild _defattr _defaultdocdir _emacslispdir _exec_prefix _findprov_default_method _findprov_lib_path _findprov_skiplist _findprov_topdir _findreq_default_method _findreq_skiplist _findreq_topdir _fixup_method _fixup_skiplist _fixup_topdir _gamesbindir _gamesdatadir _gamesdir _gnu _gzipbin _host _host_alias _host_cpu _host_os _host_vendor _iconsdir _includedir _infodir _initdir _initrddir _install_script_path _instchangelog _internal_gpg_path _lib _libdir _libexecdir _licensedir _liconsdir _localstatedir _lockdir _logdir _makeinstall_target _man1dir _man2dir _man3dir _man4dir _man5dir _man6dir _man7dir _man8dir _man9dir _mandir _menudir _miconsdir _oldincludedir _optlevel _os _package_version _perl_req_method _pgpbin _prefix _query_selector_match _repackage_dir _repackage_name_fmt _repackage_root _rpmdir _rpmfilename _sbindir _sharedstatedir _signature _smp_mflags _sourcedir _specdir _spooldir _srcrpmdir _strip_method _strip_skiplist _strip_topdir _sysconfdir _target _target_alias _target_cpu _target_os _target_platform _target_vendor _tcldatadir _tcllibdir _tmppath _topdir _topsrcdir _unzipbin _usr _usrsrc _var _vendor _verify_elf_method _verify_elf_skiplist _verify_elf_topdir _x11bindir _x11datadir _x11dir _x11fontsdir _x11includedir _x11libdir _x11mandir packagerName packagerAddress __dbi_btconfig __dbi_btconfig_current __dbi_btconfig_rebuild __dbi_htconfig __dbi_htconfig_current __dbi_htconfig_rebuild __dbi_other __dbi_perms __dbi_rebuild __dbi_transient __gcc_version __gcc_version_base __gcc_version_major __gcc_version_minor __gcc_version_patch __glibc_version __glibc_version_major __glibc_version_minor __nprocs __python_version

"some of them are essentially commands
syn region specMacroCommandLocal matchgroup=specSectionMacro start='%\(update_wms\|update_scrollkeeper\|update_menus\|uninstall_info\|strip_static\|strip_shared\|strip_relocatable\|strip_executable\|strip_binaries\|set_verify_elf_topdir\|set_verify_elf_skiplist\|set_verify_elf_method\|set_strip_topdir\|set_strip_skiplist\|set_strip_method\|set_perl_req_method\|set_libtool_version\|set_gcc_version\|set_fixup_topdir\|set_fixup_skiplist\|set_fixup_method\|set_findreq_topdir\|set_findreq_skiplist\|set_findprov_topdir\|set_findprov_skiplist\|set_compress_topdir\|set_compress_skiplist\|set_compress_method\|set_cleanup_topdir\|set_cleanup_skiplist\|set_cleanup_method\|set_automake_version\|set_autoconf_version\|remove_optflags\|preun_service\|postun_ldconfig\|post_service\|post_ldconfig_sys\|post_ldconfig_lib\|post_ldconfig\|make_session\|make_install\|make_compile\|make_build\|make\|install_info\|find_lang\|compress_docs\|cleanup_build\|clean_wms\|clean_scrollkeeper\|clean_menus\|clean_buildroot\|add_verify_elf_skiplist\|add_strip_skiplist\|add_serial\|add_optflags\|add_fixup_skiplist\|add_findreq_skiplist\|add_findprov_skiplist\|add_compress_skiplist\|add_cleanup_skiplist\|_update_wms_bin\|_update_scrollkeeper_bin\|_update_menus_bin\|_remove_install_info\|_make_bin\|_install_langs\|_install_info\|_fixperms\|_fixowner\|_fixgroup\|_defattr\|_configure_script\|__unzip\|__tar\|__subst\|__strip\|__ssh\|__spec_rmbuild_template\|__spec_rmbuild_shell\|__spec_rmbuild_pre\|__spec_rmbuild_post\|__spec_rmbuild_custom_pre\|__spec_rmbuild_custom_post\|__spec_rmbuild_cmd\|__spec_rmbuild_body\|__spec_rmbuild_args\|__spec_prep_template\|__spec_prep_shell\|__spec_prep_pre\|__spec_prep_post\|__spec_prep_custom_pre\|__spec_prep_custom_post\|__spec_prep_cmd\|__spec_prep_body\|__spec_prep_args\|__spec_install_template\|__spec_install_shell\|__spec_install_pre\|__spec_install_pre\|__spec_install_post\|__spec_install_custom_pre\|__spec_install_custom_post\|__spec_install_cmd\|__spec_install_body\|__spec_install_args\|__spec_clean_template\|__spec_clean_shell\|__spec_clean_pre\|__spec_clean_post\|__spec_clean_post\|__spec_clean_custom_pre\|__spec_clean_custom_post\|__spec_clean_cmd\|__spec_clean_body\|__spec_clean_args\|__spec_build_template\|__spec_build_shell\|__spec_build_pre\|__spec_build_post\|__spec_build_custom_pre\|__spec_build_custom_post\|__spec_build_cmd\|__spec_build_body\|__spec_build_args\|__spec_autodep_template\|__spec_autodep_shell\|__spec_autodep_pre\|__spec_autodep_post\|__spec_autodep_custom_pre\|__spec_autodep_custom_post\|__spec_autodep_cmd\|__spec_autodep_body\|__spec_autodep_args\|__sed\|__rsh\|__rm\|__remsh\|__ranlib\|__python\|__pgp_verify_cmd\|__pgp_sign_cmd\|__pgp_check_password_cmd\|__pgp5_verify_cmd\|__pgp5_sign_cmd\|__pgp5_check_password_cmd\|__pgp\|__perl\|__patch\|__os_install_post\|__objdump\|__objcopy\|__nm\|__mv\|__mkdir_p\|__mkdir\|__make\|__ln_s\|__libtoolize\|__ld\|__install_info\|__install\|__id\|__gzip\|__grep\|__gpg_verify_cmd\|__gpg_sign_cmd\|__gpg_check_password_cmd\|__gpg\|__find_requires\|__find_provides\|__file\|__cxx\|__cpp\|__cpio\|__cp\|__chown\|__chmod\|__chgrp\|__cc\|__cat\|__bzip2\|__awk\|__automake\|__autoheader\|__autoconf\|__as\|__arch_install_post\|__ar\|__aclocal\)' skip='\\$' end='$' contains=specCommandOpts,specMacroIdentifier
syn region specMacroCommandLocalBracketed matchgroup=specSectionMacro start='%{\(update_wms\|update_scrollkeeper\|update_menus\|uninstall_info\|strip_static\|strip_shared\|strip_relocatable\|strip_executable\|strip_binaries\|set_verify_elf_topdir\|set_verify_elf_skiplist\|set_verify_elf_method\|set_strip_topdir\|set_strip_skiplist\|set_strip_method\|set_perl_req_method\|set_libtool_version\|set_gcc_version\|set_fixup_topdir\|set_fixup_skiplist\|set_fixup_method\|set_findreq_topdir\|set_findreq_skiplist\|set_findprov_topdir\|set_findprov_skiplist\|set_compress_topdir\|set_compress_skiplist\|set_compress_method\|set_cleanup_topdir\|set_cleanup_skiplist\|set_cleanup_method\|set_automake_version\|set_autoconf_version\|remove_optflags\|preun_service\|postun_ldconfig\|post_service\|post_ldconfig_sys\|post_ldconfig_lib\|post_ldconfig\|make_session\|make_install\|make_compile\|make_build\|make\|install_info\|find_lang\|compress_docs\|cleanup_build\|clean_wms\|clean_scrollkeeper\|clean_menus\|clean_buildroot\|add_verify_elf_skiplist\|add_strip_skiplist\|add_serial\|add_optflags\|add_fixup_skiplist\|add_findreq_skiplist\|add_findprov_skiplist\|add_compress_skiplist\|add_cleanup_skiplist\|_update_wms_bin\|_update_scrollkeeper_bin\|_update_menus_bin\|_remove_install_info\|_make_bin\|_install_langs\|_install_info\|_fixperms\|_fixowner\|_fixgroup\|_defattr\|_configure_script\|__unzip\|__tar\|__subst\|__strip\|__ssh\|__spec_rmbuild_template\|__spec_rmbuild_shell\|__spec_rmbuild_pre\|__spec_rmbuild_post\|__spec_rmbuild_custom_pre\|__spec_rmbuild_custom_post\|__spec_rmbuild_cmd\|__spec_rmbuild_body\|__spec_rmbuild_args\|__spec_prep_template\|__spec_prep_shell\|__spec_prep_pre\|__spec_prep_post\|__spec_prep_custom_pre\|__spec_prep_custom_post\|__spec_prep_cmd\|__spec_prep_body\|__spec_prep_args\|__spec_install_template\|__spec_install_shell\|__spec_install_pre\|__spec_install_pre\|__spec_install_post\|__spec_install_custom_pre\|__spec_install_custom_post\|__spec_install_cmd\|__spec_install_body\|__spec_install_args\|__spec_clean_template\|__spec_clean_shell\|__spec_clean_pre\|__spec_clean_post\|__spec_clean_post\|__spec_clean_custom_pre\|__spec_clean_custom_post\|__spec_clean_cmd\|__spec_clean_body\|__spec_clean_args\|__spec_build_template\|__spec_build_shell\|__spec_build_pre\|__spec_build_post\|__spec_build_custom_pre\|__spec_build_custom_post\|__spec_build_cmd\|__spec_build_body\|__spec_build_args\|__spec_autodep_template\|__spec_autodep_shell\|__spec_autodep_pre\|__spec_autodep_post\|__spec_autodep_custom_pre\|__spec_autodep_custom_post\|__spec_autodep_cmd\|__spec_autodep_body\|__spec_autodep_args\|__sed\|__rsh\|__rm\|__remsh\|__ranlib\|__python\|__pgp_verify_cmd\|__pgp_sign_cmd\|__pgp_check_password_cmd\|__pgp5_verify_cmd\|__pgp5_sign_cmd\|__pgp5_check_password_cmd\|__pgp\|__perl\|__patch\|__os_install_post\|__objdump\|__objcopy\|__nm\|__mv\|__mkdir_p\|__mkdir\|__make\|__ln_s\|__libtoolize\|__ld\|__install_info\|__install\|__id\|__gzip\|__grep\|__gpg_verify_cmd\|__gpg_sign_cmd\|__gpg_check_password_cmd\|__gpg\|__find_requires\|__find_provides\|__file\|__cxx\|__cpp\|__cpio\|__cp\|__chown\|__chmod\|__chgrp\|__cc\|__cat\|__bzip2\|__awk\|__automake\|__autoheader\|__autoconf\|__as\|__arch_install_post\|__ar\|__aclocal\)\>}' skip='\\$' end='$' contains=specCommandOpts,specMacroIdentifier

"
"------------------------------------------------------------------------------
" here's is all the spec sections definitions: PreAmble, Description, Package,
"   Scripts, Files and Changelog

"One line macros - valid in all ScriptAreas
"tip: remember do include new itens on specScriptArea's skip section
syn region specSectionMacroArea matchgroup=specSectionMacro start='^%\(global\|define\|def_with\|def_without\|def_enable\|def_disable\|undefine\|patch\d*\|setup\|configure\|GNUconfigure\|find_lang\|makeinstall\)\>' skip='\\$' end='$' contains=specCommandOpts,specMacroIdentifier
syn region specSectionMacroBracketArea matchgroup=specSectionMacro start='^%{\(configure\|GNUconfigure\|find_lang\|makeinstall\)}' skip='\\$' end='$' contains=specCommandOpts,specMacroIdentifier,specIf

"%% Files Section %%
"TODO %config valid parameters: missingok\|noreplace
"TODO %verify valid parameters: \(not\)\= \(md5\|atime\|...\)
syn region specFilesArea matchgroup=specSection start='^%files\>' skip='%\(attrib\|defattr\|attr\|dir\|config\|docdir\|doc\|lang\|verify\|ghost\)\>' end='^%\(description\|package\|prep\|build\|buildmulti\|install\|clean\|pre\|postun\|preun\|post\|trigger\(in\|un\|postun\)\|files\|changelog\)\>'me=s-1 contains=specFilesOpts,specFilesDirective,@specListedFiles,specComment,specCommandSpecial,specMacroIdentifier,specIf
"tip: remember to include new itens in specFilesArea above
syn match  specFilesDirective contained '%\(attrib\|defattr\|attr\|dir\|config\|docdir\|doc\|lang\|verify\|ghost\|exclude\)\>'

"valid options for certain section headers
syn match specDescriptionOpts contained '\s-[ln]\s*\a'ms=s+1,me=e-1
syn match specPackageOpts     contained    '\s-n\s*\w'ms=s+1,me=e-1
syn match specFilesOpts       contained    '\s-f\s*\w'ms=s+1,me=e-1


syn case ignore


"%% PreAmble Section %%
"Copyright and Serial were deprecated by License and Epoch
syn region specPreAmbleDeprecated oneline matchgroup=specError start='^\(Copyright\|Serial\|BuildRoot\)' end='$' contains=specEmail,specURL,specURLMacro,specLicense,specColon,specVariables,specSpecialChar,specMacroIdentifier
syn region specPreAmble oneline matchgroup=specCommand start='^\(Prereq\|Summary\|Name\|Version\|Packager\|Requires\|Icon\|URL\|Source\d*\|Patch\d*\|Prefix\|Packager\|Group\|License\|Release\|Distribution\|Vendor\|Provides\|ExclusiveArch\|ExcludeArch\|ExclusiveOS\|Obsoletes\|BuildArch\|BuildArchitectures\|BuildRequires\|BuildConflicts\|BuildPreReq\|Conflicts\|AutoRequires\|AutoReq\|AutoReqProv\|AutoProv\|Epoch\)' end='$' contains=specEmail,specURL,specURLMacro,specLicense,specColon,specVariables,specSpecialChar,specMacroIdentifier

"%% Description Section %%
syn region specDescriptionArea matchgroup=specSection start='^%description' end='^%\(description\|package\|prep\|build\|buildmulti\|install\|clean\|pre\|postun\|preun\|post\|trigger\(in\|un\|postun\)\|files\)\>'me=s-1 contains=specDescriptionOpts,specEmail,specURL,specNumber,specAltNumber,specMacroIdentifier,specComment,specIf

"%% Package Section %%
syn region specPackageArea matchgroup=specSection start='^%package' end='^%\(description\|package\|prep\|build\|buildmulti\|install\|clean\|pre\|postun\|preun\|post\|trigger\(in\|un\|postun\)\|files\)\>'me=s-1 contains=specPackageOpts,specPreAmble,specComment,specIf

"%% Scripts Section %%
syn region specScriptArea matchgroup=specSection start='^%\(prep\|build\|buildmulti\|install\|clean\|pre\|postun\|preun\|post\|trigger\(in\|un\|postun\)\)\>' end='^%\(description\|package\|prep\|build\|buildmulti\|install\|clean\|pre\|postun\|preun\|post\|trigger\(in\|un\|postun\)\|files\|changelog\)\>'me=s-1 contains=specSpecialVariables,specVariables,@specCommands,specVariables,shDo,shFor,shCaseEsac,specNoNumberHilite,specCommandOpts,shComment,shIf,specSpecialChar,specMacroIdentifier,specSectionMacroArea,specSectionMacroBracketArea,shOperator,shQuote1,shQuote2,specMacroCommandLocal,specMacroCommandLocalBracketed,specIf

"%% Changelog Section %%
syn region specChangelogArea matchgroup=specSection start='^%changelog' end='^%\(description\|package\|prep\|build\|buildmulti\|install\|clean\|pre\|postun\|preun\|post\|trigger\(in\|un\|postun\)\|files\)\>'me=s-1 contains=specEmail,specURL,specWeekday,specMonth,specNumber,specAltNumber,specComment,specLicense


"------------------------------------------------------------------------------
"here's the shell syntax for all the Script Sections


syn case match


"sh-like comment stile, only valid in script part
syn match shComment contained '#.*$'

syn region shQuote1 contained matchgroup=shQuoteDelim start=+'+ skip=+\\'+ end=+'+ contains=specMacroIdentifier
syn region shQuote2 contained matchgroup=shQuoteDelim start=+"+ skip=+\\"+ end=+"+ contains=specVariables,specMacroIdentifier

syn match shOperator contained '[><|!&;]\|[!=]='
syn region shDo transparent matchgroup=specBlock start="\<do\>" end="\<done\>" contains=ALLBUT,shFunction,shDoError,shCase,specPreAmble,@specListedFiles

syn region specIf transparent matchgroup=specBlock start="%\(if\>\|ifdef\|ifndef\|if_without\|if_with\|if_enabled\|if_disabled\|ifosf\|ifos\|ifnos\|ifarch\|ifnarch\|else\)\>"  end='%endif'  contains=ALLBUT,specIfError,shCasea

syn region  shIf transparent matchgroup=specBlock start="\<if\>" end="\<fi\>" contains=ALLBUT,shFunction,shIfError,shCase,@specListedFiles

syn region  shFor  matchgroup=specBlock start="\<for\>" end="\<in\>" contains=ALLBUT,shFunction,shInError,shCase,@specListedFiles

syn region shCaseEsac transparent matchgroup=specBlock start="\<case\>" matchgroup=NONE end="\<in\>"me=s-1 contains=ALLBUT,shFunction,shCaseError,@specListedFiles nextgroup=shCaseEsac
syn region shCaseEsac matchgroup=specBlock start="\<in\>" end="\<esac\>" contains=ALLBUT,shFunction,shCaseError,@specListedFilesBin
syn region shCase matchgroup=specBlock contained start=")"  end=";;" contains=ALLBUT,shFunction,shCaseError,shCase,@specListedFiles

syn sync match shDoSync       grouphere  shDo       "\<do\>"
syn sync match shDoSync       groupthere shDo       "\<done\>"
syn sync match shIfSync       grouphere  shIf       "\<if\>"
syn sync match shIfSync       groupthere shIf       "\<fi\>"
syn sync match specIfSync     grouphere  specIf     "%\(if\>\|ifdef\|ifndef\|if_without\|if_with\|if_enabled\|if_disabled\|ifarch\|ifos\|ifnos\)\>"
syn sync match specIfSync     groupthere specIf     "%endif"
syn sync match shForSync      grouphere  shFor      "\<for\>"
syn sync match shForSync      groupthere shFor      "\<in\>"
syn sync match shCaseEsacSync grouphere  shCaseEsac "\<case\>"
syn sync match shCaseEsacSync groupthere shCaseEsac "\<esac\>"

" Define the default highlighting.
" For version 5.7 and earlier: only when not done already
" For version 5.8 and later: only when an item doesn't have highlighting yet
if version >= 508 || !exists("did_spec_syntax_inits")
  if version < 508
    let did_spec_syntax_inits = 1
    command -nargs=+ HiLink hi link <args>
  else
    command -nargs=+ HiLink hi def link <args>
  endif

  "main types color definitions
  HiLink specSection			Structure
  HiLink specSectionMacro		Macro
  HiLink specWWWlink			PreProc
  HiLink specOpts			Operator

  "yes, it's ugly, but white is sooo cool
  if &background == "dark"
    hi def specGlobalMacro		ctermfg=white guifg=white
  else
    HiLink specGlobalMacro		Identifier
  endif

  "sh colors
  HiLink shComment			Comment
  HiLink shIf				Statement
  HiLink shOperator			Special
  HiLink shQuote1			String
  HiLink shQuote2			String
  HiLink shQuoteDelim			Statement

  "spec colors
  HiLink specBlock			Function
  HiLink specColon			Special
  HiLink specCommand			Statement
  HiLink specMacroCommandLocal		Statement
  HiLink specCommandOpts		specOpts
  HiLink specCommandSpecial		Special
  HiLink specComment			Comment
  HiLink specConfigure			specCommand
  HiLink specDate			String
  HiLink specDescriptionOpts		specOpts
  HiLink specEmail			specWWWlink
  HiLink specError			Error
  HiLink specFilesDirective		specSectionMacro
  HiLink specFilesOpts			specOpts
  HiLink specLicense			String
  HiLink specMacroNameLocal		specGlobalMacro
  HiLink specMacroNameOther		specGlobalMacro
  HiLink specMacroIdentifier		Identifier
  HiLink specManpageFile		NONE
  HiLink specMonth			specDate
  HiLink specNoNumberHilite		NONE
  HiLink specNumber			Number
  HiLink specAltNumber			Number
  HiLink specPackageOpts		specOpts
  HiLink specPercent			Special
  HiLink specSpecialChar		Special
  HiLink specSpecialVariables		specGlobalMacro
  HiLink specSpecialVariablesNames	specGlobalMacro
  HiLink specTarCommand			specCommand
  HiLink specURL			specWWWlink
  HiLink specURLMacro			specWWWlink
  HiLink specVariables			Identifier
  HiLink specWeekday			specDate
  HiLink specListedFilesBin		Statement
  HiLink specListedFilesDoc		Statement
  HiLink specListedFilesEtc		Statement
  HiLink specListedFilesLib		Statement
  HiLink specListedFilesPrefix		Statement
  HiLink specListedFilesShare		Statement

  delcommand HiLink
endif

let b:current_syntax = "spec"

" vim: ts=8
